@@ -31,9 +31,17 @@ extern DRV_CLASS dc_nwasp;	/* NanoWasp driver */
 extern DRV_CLASS dc_myz80;	/* MYZ80 driver */
 extern DRV_CLASS dc_cfi;	/* CFI driver */
 extern DRV_CLASS dc_adisk;	/* APRIDISK driver */
+extern DRV_CLASS dc_qm;		/* CopyQM driver */
+extern DRV_CLASS dc_tele;	/* Teledisk driver */
+extern DRV_CLASS dc_logical;	/* Raw, in logical sector order */
+extern DRV_CLASS dc_rcpmfs;	/* Reverse-CP/MFS driver */
+extern DRV_CLASS dc_remote;	/* All remote drivers */
 #ifdef LINUXFLOPPY
 extern DRV_CLASS dc_linux;	/* Linux driver */
 #endif
+#ifdef NTWDMFLOPPY
+extern DRV_CLASS dc_ntwdm;	/* NT WDM driver */
+#endif
 #ifdef WIN32FLOPPY
 extern DRV_CLASS dc_win32;	/* Win32 driver */
 #endif
@@ -42,8 +50,10 @@ extern DRV_CLASS dc_win16;	/* Win16 driver */
 #endif
 #ifdef DOS32FLOPPY
 extern DRV_CLASS dc_dos32;	/* DOS32 driver */
+extern DRV_CLASS dc_dosint25;	/* DOS (INT 25h) driver */
 #endif
 #ifdef DOS16FLOPPY
 extern DRV_CLASS dc_dos16;	/* DOS16 driver */
+extern DRV_CLASS dc_dosint25;	/* DOS (INT 25h) driver */
 #endif
 
