@@ -31,8 +31,10 @@
 #include "drvi.h"
 #include "drvadisk.h"
 
+#define APRIDISK_DELETED      0xE31D0000UL
 #define APRIDISK_MAGIC        0xE31D0001UL
 #define APRIDISK_COMMENT      0xE31D0002UL
+#define APRIDISK_CREATOR           0xE31D0003UL
 #define APRIDISK_UNCOMPRESSED 0x9E90
 #define APRIDISK_COMPRESSED   0x3E5A
 
@@ -179,16 +181,28 @@ static dsk_err_t adisk_load_sector(ADISK_DSK_DRIVER *self, dsk_lsect_t nsec, FIL
 	/* Load the track header. */
 
 	err = adisk_rdlong(fp, &sh_magic); if (err) return DSK_ERR_OVERRUN;
-	if (sh_magic != APRIDISK_MAGIC && sh_magic != APRIDISK_COMMENT) 
+	if (sh_magic != APRIDISK_MAGIC 
+	&&  sh_magic != APRIDISK_COMMENT 
+	&&  sh_magic != APRIDISK_DELETED
+	&&  sh_magic != APRIDISK_CREATOR)
+	{
 		return DSK_ERR_NOTME;
+	}
 	err = adisk_rdshort(fp, &sh_compressed); if (err) return DSK_ERR_OVERRUN;
-	if (sh_compressed != APRIDISK_COMPRESSED && sh_compressed != APRIDISK_UNCOMPRESSED) return DSK_ERR_NOTME;
+	if (sh_compressed != APRIDISK_COMPRESSED 
+	&&  sh_compressed != APRIDISK_UNCOMPRESSED) 
+	{
+		return DSK_ERR_NOTME;
+	}
 	err = adisk_rdshort(fp, &sh_hdrsize);  if (err) return DSK_ERR_OVERRUN;
 	err = adisk_rdlong (fp, &sh_datasize); if (err) return DSK_ERR_OVERRUN;
 	err = adisk_rdlong (fp, &sh_identity); if (err) return DSK_ERR_OVERRUN;
 
 	/* Compressed length must be at least 3; a block can't be any smaller! */
-	if (sh_datasize < 3) return DSK_ERR_NOTME;
+	if (sh_datasize < 3 && sh_compressed == APRIDISK_COMPRESSED) 
+	{
+		return DSK_ERR_NOTME;
+	}
 	buf = dsk_malloc(1 + sh_datasize);
 	buf[sh_datasize] = 0;
 	if (buf == NULL) return DSK_ERR_NOMEM;
@@ -198,7 +212,7 @@ static dsk_err_t adisk_load_sector(ADISK_DSK_DRIVER *self, dsk_lsect_t nsec, FIL
 		if (fgetc(fp) == EOF) return DSK_ERR_OVERRUN;
 		--sh_hdrsize;
 	}
-	if (sh_magic != APRIDISK_MAGIC && sh_magic != APRIDISK_COMMENT)
+	if (sh_magic != APRIDISK_MAGIC && sh_magic != APRIDISK_COMMENT && sh_magic != APRIDISK_CREATOR)
 	{
 		while (sh_datasize > 0)
 		{
@@ -223,7 +237,16 @@ static dsk_err_t adisk_load_sector(ADISK_DSK_DRIVER *self, dsk_lsect_t nsec, FIL
 		/* APRIDISK comments for some reason are terminated by \r */	
 		for (n = 0; buf[n]; n++) 
 			if (buf[n] == '\r' && buf[n+1] != '\n') buf[n] = '\n';
-		dsk_set_comment(&self->adisk_super, buf);
+		dsk_set_comment(&self->adisk_super, (char *)buf);
+		return DSK_ERR_OK;
+	}
+	if (sh_magic == APRIDISK_CREATOR)
+	{
+		/* Don't support compressed creator blocks */
+		if (sh_compressed == APRIDISK_COMPRESSED) return DSK_ERR_OK;
+		self->adisk_creator = dsk_malloc(1 + strlen(buf));
+		if (self->adisk_creator)
+			strcpy(self->adisk_creator, buf);
 		return DSK_ERR_OK;
 	}
 	psec = &self->adisk_sectors[nsec];
@@ -231,9 +254,9 @@ static dsk_err_t adisk_load_sector(ADISK_DSK_DRIVER *self, dsk_lsect_t nsec, FIL
 	if (sh_compressed == APRIDISK_COMPRESSED)
 	{
 		/* Pass 1: Determine size of uncompressed data */
-		err = adisk_size_sector(psec, buf, sh_datasize, 0);
+		err = adisk_size_sector(psec, buf, (unsigned short)sh_datasize, 0);
 		/* Pass 2: Decompress */
-		if (!err) err = adisk_size_sector(psec, buf, sh_datasize, 1);
+		if (!err) err = adisk_size_sector(psec, buf, (unsigned short)sh_datasize, 1);
 		dsk_free(buf);
 	}
 	else
@@ -248,9 +271,9 @@ static dsk_err_t adisk_load_sector(ADISK_DSK_DRIVER *self, dsk_lsect_t nsec, FIL
 		return err;
 	}
 	psec->adisks_magic    = sh_magic;
-	psec->adisks_cylinder = sh_identity >> 16;
-	psec->adisks_head     = sh_identity & 0xFF;
-	psec->adisks_sector   = (sh_identity >> 8) & 0xFF;
+	psec->adisks_cylinder = (unsigned short)(sh_identity >> 16);
+	psec->adisks_head     = (unsigned char)(sh_identity & 0xFF);
+	psec->adisks_sector   = (unsigned char)((sh_identity >> 8) & 0xFF);
 	return DSK_ERR_OK;
 }
 
@@ -275,7 +298,8 @@ static int number_same(unsigned char *c, int tlen)
 
 static dsk_err_t adisk_save_sector(ADISK_DSK_DRIVER *self, ADISK_SECTOR *psec, FILE *fp)
 {
-	unsigned n, slen;
+	unsigned n;
+	unsigned long slen;
 	int compress = 1;
 	unsigned char *buf;
 	
@@ -301,8 +325,8 @@ static dsk_err_t adisk_save_sector(ADISK_DSK_DRIVER *self, ADISK_SECTOR *psec, F
 	{
 		buf[ 4] = APRIDISK_COMPRESSED & 0xFF;
 		buf[ 5] = APRIDISK_COMPRESSED >> 8;
-		buf[16] =  psec->adisks_datalen & 0xFF;
-		buf[17] = (psec->adisks_datalen >> 8) & 0xFF;
+		buf[16] = (unsigned char)( psec->adisks_datalen & 0xFF);
+		buf[17] = (unsigned char)((psec->adisks_datalen >> 8) & 0xFF);
 		buf[18] =  psec->adisks_data[0];
 	}
 	else
@@ -317,10 +341,10 @@ static dsk_err_t adisk_save_sector(ADISK_DSK_DRIVER *self, ADISK_SECTOR *psec, F
 	buf[ 3] =  (APRIDISK_MAGIC >> 24) & 0xFF;
 	buf[ 6] =  0x10;
 	buf[ 7] =  0;
-	buf[ 8] =  (slen - 16) & 0xFF;
-	buf[ 9] = ((slen - 16) >>  8) & 0xFF;
-	buf[10] = ((slen - 16) >> 16) & 0xFF;
-	buf[11] = ((slen - 16) >> 24) & 0xFF;
+	buf[ 8] = (unsigned char)( (slen - 16) & 0xFF);
+	buf[ 9] = (unsigned char)(((slen - 16) >>  8) & 0xFF);
+	buf[10] = (unsigned char)(((slen - 16) >> 16) & 0xFF);
+	buf[11] = (unsigned char)(((slen - 16) >> 24) & 0xFF);
 	buf[12] =  psec->adisks_head;
 	buf[13] =  psec->adisks_sector;
 	buf[14] =  psec->adisks_cylinder & 0xFF;
@@ -348,7 +372,7 @@ static dsk_err_t adisk_save_comment(ADISK_DSK_DRIVER *self, FILE *fp)
 	buf = dsk_malloc(slen = 17 + strlen(cmt));
 	if (!buf) return DSK_ERR_OK;
 	memset(buf, 0, slen);
-	strcpy(buf + 16, cmt);
+	strcpy((char *)buf + 16, cmt);
 	/* Convert lone newlines to lone CRs */
 	for (n = 17; buf[n]; n++)	 
 	{
@@ -362,10 +386,10 @@ static dsk_err_t adisk_save_comment(ADISK_DSK_DRIVER *self, FILE *fp)
 	buf[ 5] = APRIDISK_UNCOMPRESSED >> 8;
 	buf[ 6] =  0x10;
 	buf[ 7] =  0;
-	buf[ 8] =  (slen - 16) & 0xFF;
-	buf[ 9] = ((slen - 16) >>  8) & 0xFF;
-	buf[10] = ((slen - 16) >> 16) & 0xFF;
-	buf[11] = ((slen - 16) >> 24) & 0xFF;
+	buf[ 8] = (unsigned char)( (slen - 16) & 0xFF);
+	buf[ 9] = (unsigned char)(((slen - 16) >>  8) & 0xFF);
+	buf[10] = (unsigned char)(((slen - 16) >> 16) & 0xFF);
+	buf[11] = (unsigned char)(((slen - 16) >> 24) & 0xFF);
 
 	if (fwrite(buf, 1, slen, fp) < slen)
 	{
@@ -377,6 +401,53 @@ static dsk_err_t adisk_save_comment(ADISK_DSK_DRIVER *self, FILE *fp)
 }
 
 
+
+static dsk_err_t adisk_save_creator(ADISK_DSK_DRIVER *self, FILE *fp)
+{
+	char *cmt = NULL;
+	unsigned char *buf;
+	unsigned long slen;
+	int n;
+
+/*
+	if (self->adisk_creator) cmt = self->adisk_creator; 
+	else
+*/
+	cmt = "LIBDSK v" LIBDSK_VERSION;
+	
+	buf = dsk_malloc(slen = 17 + strlen(cmt));
+	if (!buf) return DSK_ERR_OK;
+	memset(buf, 0, slen);
+	strcpy((char *)buf + 16, cmt);
+	/* Convert lone newlines to lone CRs */
+	for (n = 17; buf[n]; n++)	 
+	{
+		if (buf[n] == '\n' && buf[n-1] != '\r') buf[n] = '\r';
+	}	
+	buf[ 0] =   APRIDISK_CREATOR        & 0xFF;
+	buf[ 1] =  (APRIDISK_CREATOR >> 8 ) & 0xFF;
+	buf[ 2] =  (APRIDISK_CREATOR >> 16) & 0xFF;
+	buf[ 3] =  (APRIDISK_CREATOR >> 24) & 0xFF;
+	buf[ 4] = APRIDISK_UNCOMPRESSED & 0xFF;
+	buf[ 5] = APRIDISK_UNCOMPRESSED >> 8;
+	buf[ 6] =  0x10;
+	buf[ 7] =  0;
+	buf[ 8] = (unsigned char)( (slen - 16) & 0xFF);
+	buf[ 9] = (unsigned char)(((slen - 16) >>  8) & 0xFF);
+	buf[10] = (unsigned char)(((slen - 16) >> 16) & 0xFF);
+	buf[11] = (unsigned char)(((slen - 16) >> 24) & 0xFF);
+
+	if (fwrite(buf, 1, slen, fp) < slen)
+	{
+		dsk_free(buf);
+		return DSK_ERR_SYSERR;
+	}
+	dsk_free(buf);
+	return DSK_ERR_OK;
+}
+
+
+
 dsk_err_t adisk_open(DSK_DRIVER *self, const char *filename)
 {
 	FILE *fp;
@@ -398,9 +469,14 @@ dsk_err_t adisk_open(DSK_DRIVER *self, const char *filename)
 	if (!fp) return DSK_ERR_NOTME;
 
 	/* Try to check the magic number at 0x80 */
-	if (fread(adiskself->adisk_header, 1, 
-		sizeof(adiskself->adisk_header), fp) < sizeof(adiskself->adisk_header) 
-	|| (adisk_rdlong(fp, &magic) != DSK_ERR_OK) || (magic != APRIDISK_MAGIC))
+	if (fread(adiskself->adisk_header, 1, sizeof(adiskself->adisk_header), 
+				fp) < sizeof(adiskself->adisk_header) 
+	|| (memcmp(adiskself->adisk_header, adisk_wmagic, sizeof(adisk_wmagic)))
+	|| (adisk_rdlong(fp, &magic) != DSK_ERR_OK) 
+	|| (magic != APRIDISK_MAGIC && 
+	    magic != APRIDISK_CREATOR &&
+	    magic != APRIDISK_COMMENT &&
+	    magic != APRIDISK_DELETED))
 	{
 		fclose(fp);
 		return DSK_ERR_NOTME;
@@ -499,7 +575,8 @@ dsk_err_t adisk_close(DSK_DRIVER *self)
 			if (fwrite(adisk_wmagic, 1, 128, fp) < 128)
 			{
 				err = DSK_ERR_SYSERR;
-			}	
+			}
+			else if ((err = adisk_save_creator(adiskself, fp))) { }
 			else for (trk = 0; trk < adiskself->adisk_maxsectors; trk++)
 			{
 				err = adisk_save_sector(adiskself, 
@@ -528,6 +605,11 @@ dsk_err_t adisk_close(DSK_DRIVER *self)
 		dsk_free(adiskself->adisk_filename);
 		adiskself->adisk_filename = NULL;
 	}
+	if (adiskself->adisk_creator)
+	{
+		dsk_free(adiskself->adisk_creator);
+		adiskself->adisk_creator = NULL;
+	}
 	return err;
 }
 
@@ -608,7 +690,7 @@ dsk_err_t adisk_format(DSK_DRIVER *self, DSK_GEOMETRY *geom,
 {
 	ADISK_DSK_DRIVER *adiskself;
 	dsk_err_t err;
-	int nlsec;
+	dsk_psect_t nlsec;
 	unsigned long nsec, secsize;
 	void *secbuf;
 	ADISK_SECTOR *psec;
@@ -719,7 +801,7 @@ dsk_err_t adisk_secid(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 	 * Set lastmatch to be the last one found. 
 	 * If zeromatch=lastmatch, reset the count so that the next
 	 * call returns the first sector */
-	for (nsec = 0; nsec < adiskself->adisk_nsectors; nsec++)
+	for (nsec = 0; nsec < (int)adiskself->adisk_nsectors; nsec++)
 	{
 		if (adiskself->adisk_sectors[nsec].adisks_cylinder == cylinder &&
 		    adiskself->adisk_sectors[nsec].adisks_head     == head &&
@@ -748,3 +830,5 @@ dsk_err_t adisk_secid(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 	
 	return DSK_ERR_OK;
 }
+
+
