@@ -425,11 +425,11 @@ dsk_err_t win32_read(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
         res = ReadFile(w32self->w32_hdisk, iobuffer, geom->dg_secsize, &bytesread, NULL);
         if (!res)
         {
-            VirtualFree(iobuffer, geom->dg_secsize, MEM_RELEASE);
+            VirtualFree(iobuffer, 0, MEM_RELEASE);
        	    return translate_win32_error();
         }
         memcpy(buf, iobuffer, geom->dg_secsize);
-        VirtualFree(iobuffer, geom->dg_secsize, MEM_RELEASE);
+        VirtualFree(iobuffer, 0, MEM_RELEASE);
 
         if (bytesread < geom->dg_secsize)
         {
@@ -479,10 +479,10 @@ dsk_err_t win32_write(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
         res = WriteFile(w32self->w32_hdisk, iobuffer, geom->dg_secsize, &byteswritten, NULL);
         if (!res || (byteswritten < (unsigned)geom->dg_secsize))
         {
-            VirtualFree(iobuffer, geom->dg_secsize, MEM_RELEASE);
+            VirtualFree(iobuffer, 0, MEM_RELEASE);
        	    return translate_win32_error();
         }
-        VirtualFree(iobuffer, geom->dg_secsize, MEM_RELEASE);
+        VirtualFree(iobuffer, 0, MEM_RELEASE);
 	return DSK_ERR_OK;
 }
 
